require 'xmlrpc/client' 
require 'fileutils'
require 'pp'

class PathCache
  include FileUtils


  def self.for_this_test(test_filename)
    new("path-cache/#{test_filename}")
  end

  def initialize(dir)
    @dir = dir
    @server = XMLRPC::Client.new("localhost", "/side-channel", 8080)
  end

  def remember_path(snapshot_file=nil)
    snapshot_file ||= caller_file_in(@dir)
    mkdir_p @dir
    File.open(snapshot_file, 'w') { | io | 
      io.puts @server.call("snapshot")
    }
  end

  def has_path?(snapshot_file=nil)
    snapshot_file ||= caller_file_in(@dir)
    File.exist?(snapshot_file)
  end

  def replay_path(snapshot_file=nil)
    snapshot_file ||= caller_file_in(@dir)
    commands = IO.read(snapshot_file)
    @server.call('replay', commands)
  end


  private
  
  def caller_file_in(dir)
    unless caller[2] =~ /`(.+)'/
      raise StandardError.new("bad call stack: #{caller}")
    end
    calling_method = $1
    "#{dir}/#{calling_method}"
  end
end



